<?php
if (!defined('PLX_ROOT')) { exit; };

/*
 * Plugin pour HighlightJS
 * https://github.com/highlightjs/highlight.js
 *
 * Permet la coloration syntaxique d'un extrait de code.
 * L'extrait doit être placé entre la paire de couples de balises :
 *
 * <pre><code lang="hljs php"> et </code></pre>
 *
 * Replace php with an other language if needed.
 * Available languages in common usage :
 * */

class kzHighlightJS extends plxPlugin {

	const ROOT = '/highlight.js/build/';
	const URL = PLX_PLUGINS . __CLASS__ . self::ROOT;
	const SCRIPT_SRC = PLX_PLUGINS . __CLASS__ . '/' . __CLASS__ . '.js';
	const STYLES_LIST = __DIR__ . '/styles.txt';
	const PARAM1 = 'theme';

	/* Replace each '<' by entity inside <pre><code> tags in the content of an article or a static page */
	/* escaping '<' and '?>' avoids running PHP inside <pre><code> blocks in static pages  */
	const _PATTERN = <<< '_PATTERN'
'#(<pre[^>]*>\s*<code[^>]*>)\s*(.*?)\s*(</code>\s*</pre>)#is'
_PATTERN;
	const _REPLACE = <<< '_REPLACE'
function($m) { return $m[1].strtr($m[2], array('<' => '&lt;', '?>' => '?&gt;')).$m[3]; }
_REPLACE;
	const _PARSE_ARTICLE_BEGIN = <<< '_PARSE_ARTICLE_BEGIN'
<?php
$art['content'] = preg_replace_callback(
_PARSE_ARTICLE_BEGIN;
	const _PARSE_ARTICLE_END = <<< '_PARSE_ARTICLE_END'
	$art['content']
);
?>
_PARSE_ARTICLE_END;
	const PARSE_ARTICLE_CODE =
		self::_PARSE_ARTICLE_BEGIN ."\n\t" .
		self::_PATTERN . ",\n\t" .
		self::_REPLACE . ",\n\t" .
		self::_PARSE_ARTICLE_END;

	/* same play with a static page */
	const _PARSE_STATIC_PART1 = <<< '_PARSE_STATIC_PART1'
<?php
if($this->plxMotor->aStats[ $this->plxMotor->cible ]['readable'] != 1) {
	echo '<p>'.L_STATICCONTENT_INPROCESS.'</p>';
	return true;
} else {
	$pattern =
_PARSE_STATIC_PART1;
	const _PARSE_STATIC_PART2 = <<< '_PARSE_STATIC_PART2'
	$file =
		PLX_ROOT .
		$this->plxMotor->aConf['racine_statiques'] .
		$this->plxMotor->cible . '.' .
		$this->plxMotor->aStats[ $this->plxMotor->cible ]['url'] .
		'.php';
	$content = file_get_contents($file);
	if(preg_match($pattern, $content)) {
		$new_content = preg_replace_callback(
			$pattern
_PARSE_STATIC_PART2;
	// In eval() expression, spaces are required for php tags
	const _PARSE_STATIC_PART3 = <<< '_PARSE_STATIC_PART3'
$content
		);
		ob_start();
		eval(' ?>'.$new_content.'<?php ');
		$output = ob_get_clean();
		eval($this->plxMotor->plxPlugins->callHook('plxShowStaticContent'));
		echo $output;
		return true;
	}
	return false;
}
?>
_PARSE_STATIC_PART3;
	const PARSE_STATIC_CODE =
		self::_PARSE_STATIC_PART1 .
		self::_PATTERN . ";\n" .
		self::_PARSE_STATIC_PART2 . ",\n\t\t\t" .
		self::_REPLACE . ",\n\t\t\t" .
		self::_PARSE_STATIC_PART3;

	public $themes = false;

	public function __construct($default_lang) {
		parent::__construct($default_lang);
		parent::setConfigProfil(PROFIL_ADMIN);
		if(!defined('PLX_ADMIN')) {
			parent::addHook('ThemeEndHead', 'ThemeEndHead');
	        parent::addHook('ThemeEndBody', 'ThemeEndBody');
	        parent::addHook('plxMotorParseArticle', 'plxMotorParseArticle');
	        parent::addHook('plxShowStaticContentBegin', 'plxShowStaticContentBegin');
		} elseif(basename($_SERVER['REQUEST_URI']) == 'parametres_plugin.php?p=' . __CLASS__) {
			// Just use for parametres_plugin.php in admin mode
			$this->themes = array();

			if(file_exists(self::STYLES_LIST)) {
				foreach(file(self::STYLES_LIST, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) as $k) {
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
			}

			if(empty($this->themes)) {
				foreach(glob(__DIR__ . self::ROOT.'/demo/styles/*.css') as $t) {
					$k = basename($t, '.css');
					$this->themes[$k] = ucwords(str_replace('-', ' ', $k));
				}
				if(is_writable(__DIR__)) {
					file_put_contents(self::STYLES_LIST, implode(PHP_EOL, array_keys($this->themes)));
				}
			}
			parent::addHook('AdminTopEndHead', 'AdminTopEndHead');
			parent::addHook('AdminFootEndBody', 'ThemeEndBody');
		}
	}

	public function OnActivate() {
		// Lists again all available themes
		if(file_exists(self::STYLES_LIST) and is_writable(__DIR__)) {
			unlink(self::STYLES_LIST);
		}
	}

	/*
	public function OnDeActivate() { }
	public function onUpdate() { }
	 * */

	public function plxMotorParseArticle() {
		// Hack against PluXml when publishing;
		echo self::PARSE_ARTICLE_CODE;
	}

	public function plxShowStaticContentBegin() {
		// escape '<' for content inside <pre><code> tags for a static page
		echo self::PARSE_STATIC_CODE;
	}

	public function AdminTopEndHead() {
		// using alternate attribute avoids for loading all stylesheets if not required
		$currentStyle = $this->getParam(self::PARAM1);
		if(empty($currentStyle)) { $currentStyle = 'default'; }
		foreach($this->themes as $k=>$v) {
			$href = self::URL."demo/styles/$k.css";
?>
	<link rel="<?php if($k != $currentStyle) echo 'alternate '; ?>stylesheet" type="text/css" href="<?php echo $href; ?>" title="<?php echo $k; ?>" disabled />
<?php
		}
	}

	public function ThemeEndHead() {
		// Uncomment for debugging
		// file_put_contents('data/test-article.html', self::PARSE_ART_CODE);
		// file_put_contents('data/test-static.php', self::PARSE_STATIC_CODE);
		$param = trim($this->getParam(self::PARAM1));
		if(empty($param)) { $param = 'default'; }
		$href = 'demo/styles/'.$param.'.css';
		if(file_exists(__DIR__ . self::ROOT . $href)) {
?>
	<link rel="stylesheet" type="text/css" href="<?php echo self::URL . $href; ?>" />
<?php
		}
	}

	public function ThemeEndBody() {
		// https://highlightjs.org/usage/
		// Also in use for config.php for selecting a theme

		$t = glob(__DIR__ . self::ROOT . 'demo/jquery*.js');
		if(empty($t)) {
			$folder = self::ROOT . 'demo/';
			echo <<< EOT
<!--
jQuery not found in $folder
-->
EOT;
			return;
		}
		$src = substr(self::ROOT, 1) . 'demo/' . basename($t[0]);
?>
	<script type="text/javascript" src="<?php echo self::URL; ?>highlight.pack.js"></script>
	<script type="text/javascript" src="<?php echo self::SCRIPT_SRC; ?>" data-src="<?php echo $src; ?>"></script>
<?php
	}

}
?>
